; Hello World for SNES

; Uses Mode 0

; use the cc65 assembler
;   cl65 -t none -o hello_world.o -l hello_world.lst -c hello_world.s
;   ld65 -o hello_world.sfc --config hello_world.cfg --obj hello_world.o

.setcpu		"65816"

.segment "STARTUP"

.proc	InitRegs
	sep	#$20	; set processor bit M (16-bit mode)
	lda	#$8f	; put 0x8f in accumulator
			; (screen off, full brightness)
	sta	$2100	; store to brightness/screen reg

	stz	$2101	; sprite register (size+address, VRAM)
	stz	$2102	; sprite register (address in mem, OAM)
	stz	$2103	; "
	stz	$2105	; graphic mode register (mode 0)
	stz	$2106	; mosaic register (noplanes, nomosaic)
	stz	$2107	; plane 0 map VRAM location
	stz	$2108	; plane 1 map VRAM location
	stz	$2109	; plane 2 map VRAM location
	stz	$210a	; plane 3 map VRAM location
	stz	$210b	; plane 0+1 tile location
	stz	$210c	; plane 2+3 tile location
	stz	$210d	; Plane 0 scroll x (first 8 bits)
	stz	$210d	; Plane 0 scroll x (last 3 bits) #$0 - #$07ff
	stz	$210e	; Plane 0 scroll y (first 8 bits)
	stz	$210e	; Plane 0 scroll y (last 3 bits) #$0 - #$07ff
	stz	$210f	; Plane 1 scroll x (first 8 bits)
	stz	$210f	; Plane 1 scroll x (last 3 bits) #$0 - #$07ff
	stz	$2110	; Plane 1 scroll y (first 8 bits)
	stz	$2110	; Plane 1 scroll y (last 3 bits) #$0 - #$07ff
	stz	$2111	; Plane 2 scroll x (first 8 bits)
	stz	$2111	; Plane 2 scroll x (last 3 bits) #$0 - #$07ff
	stz	$2112	; Plane 2 scroll y (first 8 bits)
	stz	$2112	; Plane 2 scroll y (last 3 bits) #$0 - #$07ff
	stz	$2113	; Plane 3 scroll x (first 8 bits)
	stz	$2113	; Plane 3 scroll x (last 3 bits) #$0 - #$07ff
	stz	$2114	; Plane 3 scroll y (first 8 bits)
	stz	$2114	; Plane 3 scroll y (last 3 bits) #$0 - #$07ff

	lda	#$80	; increase VRAM address after writing to $2119
	sta	$2115	; VRAM address increment register
	stz	$2116	; VRAM address low
	stz	$2117	; VRAM address high
	stz	$211a	; Initial Mode 7 setting register
	stz	$211b	; Mode 7 matrix parameter A register (low)

	lda	#$01
	sta	$211b	; Mode 7 matrix parameter A register (high)
	stz	$211c	; Mode 7 matrix parameter B register (low)
	stz	$211c	; Mode 7 matrix parameter B register (high)
	stz	$211d	; Mode 7 matrix parameter C register (low)
	stz	$211d	; Mode 7 matrix parameter C register (high)
	stz	$211e	; Mode 7 matrix parameter D register (low)
	stz	$211e	; Mode 7 matrix parameter D register (high)
	stz	$211f	; Mode 7 center position X register (low)
	stz	$211f	; Mode 7 center position X register (high)
	stz	$2120	; Mode 7 center position Y register (low)
	stz	$2120	; Mode 7 center position Y register (high)

	stz	$2121	; color # register
	stz	$2123	; bg1 & bg2 window mask reg
	stz	$2124	; bg3 & bg4 window mask reg
	stz	$2125	; obj & color mask reg
	stz	$2126	; window 1 left pos
	stz	$2127	; window 2 left pos
	stz	$2128	; window 3 left pos
	stz	$2129	; window 4 left pos
	stz	$212a	; bg1,2,3,4 window logic reg
	stz	$212b	; obj color win logic reg (or, and, xor, nor)

	lda	#$01	;
	sta	$212c	; main screen desig (plane, sprite enable)
	stz	$212d	; sub screen desig
	stz	$212e	; window mask main screen
	stz	$212f	; window mask sub screen
	lda	#$30	;
	sta	$2130	; color addition and screen addition
	stz	$2131	; add/sub desig for screen/sprite/color
	lda	#$e0	;
	sta	$2132	; color data for add/sub
	stz	$2133	; screen setting (interlace, x,y enable,  SFX data)

	stz	$4200	; enable v-blank interrupt, joypad register
	lda	#$ff	;
	sta	$4201	; programmable i/o port
	stz	$4202	; multiplicand A
	stz	$4203	; multiplier B
	stz	$4204	; multiplier C
	stz	$4205	; multiplicand C
	stz	$4206	; Divisor B
	stz	$4207	; Horizontal Count Timer
	stz	$4208	; Horizontal Count MSB
	stz	$4209	; Vertical Count Timer
	stz	$420a	; Vertical Count MSB
	stz	$420b	; General DMA enable
	stz	$420c	; Horizontal DMA enable
	stz	$420d	; Access cycle designation (slow/fast ROM)
	;cli		; enable interrupts
	rts
.endproc


;
Palette:
.include "tb1.pal"

;
Pattern:

.include "tb1.font"

String:
;.byte 65
;.byte $20,$21,$22,$23
	.asciiz	"HELLO,_WORLD!"

;
; NVMXDIZC
; Neg oVflo Mem=8 Xixiy=8 Decimal Interrupt Zero Carry

.proc	Reset

	sei	; disable interrupts
	clc	; clear carry
	xce	; and exchange with X to enable native mode
	phk	; push program bank on stack?
	plb	; pull data bank from stack?

	rep	#$38	; clear status bits, binary mode, A and IX/IY = 16
.i16			; tell assembler IX/IY=16bits
	ldx	#$1fff	; set the stack pointer to be  0x1fff
	txs		; move X to stack

	jsr	InitRegs	; call to initregs

	sep	#$20		; set accumulator to 8-bit
.a8				; tell assembler A=8bits

	lda	#$40
	sta	$2107		; bg1 src?
	stz	$210b		; bg character data area
	
; Copy Palettes
	stz	$2121		; CGRAM color-generator read/write address
	ldy	#$0200		; counter, 512 bytes
	ldx	#$0000		; pointer
copypal:
	lda	Palette, x	; load byte of palette
	sta	$2122		; store to color generator
	inx
	dey
	bne	copypal

; Copy Patterns
	rep	#$20		; set accumulator to 16bit
.a16
	lda	#$0000		; 
	sta	$2116		; set adddress for VRAM read/write
	ldy	#$2000		; 8kB?
	ldx	#$0000
copyptn:
	lda	Pattern, x
	sta	$2118		; write the data
	inx
	inx
	dey
	bne	copyptn

; Copy NameTable
	lda	#$41a9		; 
	sta	$2116		; set VRAM r/w address
	ldy	#$000d
	ldx	#$0000
	lda	#$0000
copyname:
	txa
	asl
	asl
	asl
	asl
	asl
	asl
	asl
	asl
	asl
	asl
	
		
	sep	#$20		; set accumulator to 8 bit
	
.a8

	lda	String, x	; load string

	rep	#$20		; set accumulator to 16 bit?
.a16
	sta	$2118		; store to VRAM
	                        ; the bottom 8 bits is the tile to use
				; the top 8 bits is vhopppcc
				; vert flip, horiz flip o=priority
				; p = palette, c=top bits of tile#
	inx
		
	dey
	bne	copyname

	lda	#$01
	sta	$212c		; Display bg1 as main screen
	stz	$212d		; turn off subscreen
	lda	#$0f
	sta	$2100		; unblank screen
mainloop:
	jmp	mainloop

	rti
.endproc

; 
.segment "CARTINFO"
	.byte	"SAMPLE1               "	; Game Title
	.byte	$01				; 0x01:HiRom, 0x30:FastRom(3.57MHz)
	.byte	$05				; ROM Size (2KByte * N)
	.byte	$00				; RAM Size (8KByte * N)
	.word	$0001				; Developper ID ?
	.byte	$00				; Version
	.byte	$7f, $73, $80, $8c		; Security Key ?
	.byte	$ff, $ff, $ff, $ff		; Security Key ?

	.word	$0000	; Native:COP
	.word	$0000	; Native:BRK
	.word	$0000	; Native:ABORT
	.word	$0000	; Native:NMI
	.word	$0000	; 
	.word	$0000	; Native:IRQ

	.word	$0000	; 
	.word	$0000	; 

	.word	$0000	; Emulation:COP
	.word	$0000	; 
	.word	$0000	; Emulation:ABORT
	.word	$0000	; Emulation:NMI
	.word	Reset	; Emulation:RESET
	.word	$0000	; Emulation:IRQ/BRK
